
'''
	import pathlib
	THIS_FOLDER = pathlib.Path (__file__).parent.resolve ()

	from os.path import dirname, join, normpath
	SEARCH = normpath (join (THIS_FOLDER, "../.."))

	import BODY_SCAN
	BODY_SCAN.START (
		GLOB 		= SEARCH + '/**/*STATUS.py'
	)
'''

import glob

from BOTANIST.PORTS.FIND_AN_OPEN_PORT import FIND_AN_OPEN_PORT

from .START_MULTIPLE_PROCESSES import START_MULTIPLE_PROCESSES

def CHECK_STATUS_LOCATION ():
	import pathlib
	THIS_FOLDER = pathlib.Path (__file__).parent.resolve ()

	from os.path import dirname, join, normpath
	CHECK_STATUS = normpath (join (THIS_FOLDER, "SCAN/__init__.py"))
	
	return CHECK_STATUS

def START (
	GLOB = "",
	RELATIVE_PATH = False,
	
	MODULE_PATHS = []
	
	#DB_FOLDER 	= DB_FOLDER
):
	FINDS = glob.glob (GLOB, recursive = True)
	
	START_PORT = 40000;
	PORT = START_PORT;
	
	print ()
	print ("SEARCHING FOR GLOB:")
	print ("	", GLOB)
	print ()
	
	PORTS = []
	PROCESSES = []
	for FOUND in FINDS:
		print ("	FOUND:", FOUND)
		
		PORTS.append (PORT)
		
		PROCESSES.append ({
			"STRING": f'python3 { CHECK_STATUS_LOCATION () }  KEG OPEN --port { PORT }',
			"CWD": None
		})
		
		PORT += 1;
		
	print ()
	print ("	FINDS:", FINDS)
	print ("	FINDS COUNT:", len (FINDS))
	print ();

	PROCS = START_MULTIPLE_PROCESSES (
		PROCESSES = PROCESSES
	)
	
	import time
	time.sleep (0.5)
	
	REQUEST_ADDRESS = f'http://127.0.0.1:{ PORTS [0] }'
	
	import json
	import requests
	r = requests.put (
		REQUEST_ADDRESS, 
		data = json.dumps ({ 
			"FINDS": FINDS,
			"MODULE PATHS": MODULE_PATHS,
			"RELATIVE PATH": RELATIVE_PATH
		})
	)
	
	def FORMAT_RESPONSE (TEXT):
		import json
		return json.loads (TEXT)
	
	STATUS = FORMAT_RESPONSE (r.text)
	
	print ()
	print ("REQUEST ADDRESS :", REQUEST_ADDRESS)
	print ("REQUEST STATUS  :", r.status_code)
	print ("REQUEST TEXT  :", json.dumps (STATUS, indent = 4))
	print ()
	
	EXIT 			= PROCS ["EXIT"]
	PROCESSES 		= PROCS ["PROCESSES"]

	#time.sleep (2.5)
	
	return {
		"STATUS": STATUS
	}
	
	#EXIT ()




def ADD_PATHS_TO_SYSTEM (PATHS):
	import pathlib
	FIELD = pathlib.Path (__file__).parent.resolve ()

	from os.path import dirname, join, normpath
	import sys
	for PATH in PATHS:
		sys.path.insert (0, normpath (join (FIELD, PATH)))

from .SCAN import SCAN

import json

def TAP (
	PORT = 0
):
	print ("OPENING KEG ON PORT:", PORT)

	from flask import Flask, request

	app = Flask (__name__)

	@app.route ("/", methods = [ 'GET' ])
	def HOME ():	
		return "?"

	@app.route ("/", methods = [ 'PUT' ])
	def HOME_POST ():
		print ("@ HOME PUT", request.data)
	
		DATA = json.loads (request.data.decode ('utf8'))
		print ("DATA:", DATA)

		FINDS = DATA ['FINDS']
		MODULE_PATHS = DATA ['MODULE PATHS']
		RELATIVE_PATH = DATA ['RELATIVE PATH']


		ADD_PATHS_TO_SYSTEM (MODULE_PATHS)

		STATUS = {
			"PATHS": [],
			"STATS": {
				"EMPTY": 0,
				"CHECKS": {
					"PASSES": 0,
					"ALARMS": 0
				}
			}
		}

		for FIND in FINDS:
			SCAN_STATUS = SCAN (FIND)
				
			if ("EMPTY" in SCAN_STATUS and SCAN_STATUS ["EMPTY"] == True):
				STATUS ["STATS"] ["EMPTY"] += 1

			try:
				STATUS ["STATS"] ["CHECKS"] ["PASSES"] += SCAN_STATUS ["STATS"] ["PASSES"]
			except Exception:
				pass;
			
			try:
				STATUS ["STATS"] ["CHECKS"] ["ALARMS"] +=	SCAN_STATUS	["STATS"] ["ALARMS"]
			except Exception:
				pass;
		
			import os
			if (type (RELATIVE_PATH) == str):
				PATH = os.path.relpath (FIND, RELATIVE_PATH)
			else:
				PATH = FIND
			
			STATUS ["PATHS"].append ({
				"PATH": PATH,
				** SCAN_STATUS
			})
			
			
		return json.dumps (STATUS, indent = 4)
		
	app.run (
		port = PORT
	)